@@ -19,22 +19,22 @@ subset(x, doc_id == 100)
 ## -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 library(data.table)
 x <- as.data.table(x)
-x <- x[, pos_previous   := shift(pos, n = 1, type = "lag"), by = list(doc_id)]
-x <- x[, pos_next       := shift(pos, n = 1, type = "lead"), by = list(doc_id)]
-x <- x[, token_previous := shift(token, n = 1, type = "lag"), by = list(doc_id)]
+x <- x[, pos_previous   := shift(pos, n = 1, type = "lag"),    by = list(doc_id)]
+x <- x[, pos_next       := shift(pos, n = 1, type = "lead"),   by = list(doc_id)]
+x <- x[, token_previous := shift(token, n = 1, type = "lag"),  by = list(doc_id)]
 x <- x[, token_next     := shift(token, n = 1, type = "lead"), by = list(doc_id)]
 
 ## -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
-x <- x[, pos_previous   := txt_sprintf("pos[w-1]=%s", pos_previous), by = list(doc_id)]
-x <- x[, pos_next       := txt_sprintf("pos[w+1]=%s", pos_next), by = list(doc_id)]
+x <- x[, pos_previous   := txt_sprintf("pos[w-1]=%s", pos_previous),     by = list(doc_id)]
+x <- x[, pos_next       := txt_sprintf("pos[w+1]=%s", pos_next),         by = list(doc_id)]
 x <- x[, token_previous := txt_sprintf("token[w-1]=%s", token_previous), by = list(doc_id)]
-x <- x[, token_next     := txt_sprintf("token[w-1]=%s", token_next), by = list(doc_id)]
+x <- x[, token_next     := txt_sprintf("token[w-1]=%s", token_next),     by = list(doc_id)]
 subset(x, doc_id == 100, select = c("doc_id", "token", "token_previous", "token_next"))
 x <- as.data.frame(x)
 
 ## -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 crf_train <- subset(x, data == "ned.train")
-crf_test <- subset(x, data == "testa")
+crf_test  <- subset(x, data == "testa")
 
 ## -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 model <- crf(y = crf_train$label, 
@@ -48,6 +48,11 @@ stats <- summary(model)
 plot(stats$iterations$loss, pch = 20, type = "b", 
      main = "Loss evolution", xlab = "Iteration", ylab = "Loss")
 
+## -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
+weights     <- coefficients(model, encoding = "UTF-8")
+head(weights$transitions)
+head(subset(weights$states, label %in% "B-LOC"), n = 10)
+
 ## -------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------------
 scores <- predict(model, 
                   newdata = crf_test[, c("pos", "pos_previous", "pos_next", 
